@@ -1201,7 +1201,7 @@ dpif_execute_helper_cb(void *aux_, struct dp_packet_batch *packets_,
              * that we supply as metadata.  We have to use a "set" action to
              * supply it. */
             if (md->tunnel.ip_dst) {
-                odp_put_tunnel_action(&md->tunnel, &execute_actions);
+                odp_put_tunnel_action(&md->tunnel, &execute_actions, NULL);
             }
             ofpbuf_put(&execute_actions, action, NLA_ALIGN(action->nla_len));
 
