@@ -510,8 +510,8 @@ nx_pull_match_entry(struct ofpbuf *b, bool allow_cookie,
  * ethertype being present, when decoding metadata only. */
 static enum ofperr
 nx_pull_raw(const uint8_t *p, unsigned int match_len, bool strict,
-            struct match *match, ovs_be64 *cookie, ovs_be64 *cookie_mask,
-            const struct tun_table *tun_table,
+            bool pipeline_fields_only, struct match *match, ovs_be64 *cookie,
+            ovs_be64 *cookie_mask, const struct tun_table *tun_table,
             const struct vl_mff_map *vl_mff_map)
 {
     ovs_assert((cookie != NULL) == (cookie_mask != NULL));
@@ -545,10 +545,12 @@ nx_pull_raw(const uint8_t *p, unsigned int match_len, bool strict,
                 *cookie = value.be64;
                 *cookie_mask = mask.be64;
             }
-        } else if (strict && !mf_are_prereqs_ok(field, &match->flow, NULL)) {
+        } else if (strict && !mf_are_match_prereqs_ok(field, match)) {
             error = OFPERR_OFPBMC_BAD_PREREQ;
         } else if (!mf_is_all_wild(field, &match->wc)) {
             error = OFPERR_OFPBMC_DUP_FIELD;
+        } else if (pipeline_fields_only && !mf_is_pipeline_field(field)) {
+            error = OFPERR_OFPBRC_PIPELINE_FIELDS_ONLY;
         } else {
             char *err_str;
 
@@ -559,6 +561,8 @@ nx_pull_raw(const uint8_t *p, unsigned int match_len, bool strict,
                 free(err_str);
                 return OFPERR_OFPBMC_BAD_VALUE;
             }
+
+            match_add_ethernet_prereq(match, field);
         }
 
         if (error) {
@@ -575,7 +579,7 @@ nx_pull_raw(const uint8_t *p, unsigned int match_len, bool strict,
 
 static enum ofperr
 nx_pull_match__(struct ofpbuf *b, unsigned int match_len, bool strict,
-                struct match *match,
+                bool pipeline_fields_only, struct match *match,
                 ovs_be64 *cookie, ovs_be64 *cookie_mask,
                 const struct tun_table *tun_table,
                 const struct vl_mff_map *vl_mff_map)
@@ -592,14 +596,17 @@ nx_pull_match__(struct ofpbuf *b, unsigned int match_len, bool strict,
         }
     }
 
-    return nx_pull_raw(p, match_len, strict, match, cookie, cookie_mask,
-                       tun_table, vl_mff_map);
+    return nx_pull_raw(p, match_len, strict, pipeline_fields_only, match,
+                       cookie, cookie_mask, tun_table, vl_mff_map);
 }
 
 /* Parses the nx_match formatted match description in 'b' with length
  * 'match_len'.  Stores the results in 'match'.  If 'cookie' and 'cookie_mask'
  * are valid pointers, then stores the cookie and mask in them if 'b' contains
  * a "NXM_NX_COOKIE*" match.  Otherwise, stores 0 in both.
+ * If 'pipeline_fields_only' is true, this function returns
+ * OFPERR_OFPBRC_PIPELINE_FIELDS_ONLY if there is any non pipeline fields
+ * in 'b'.
  *
  * 'vl_mff_map" is an optional parameter that is used to validate the length
  * of variable length mf_fields in 'match'. If it is not provided, the
@@ -611,28 +618,28 @@ nx_pull_match__(struct ofpbuf *b, unsigned int match_len, bool strict,
 enum ofperr
 nx_pull_match(struct ofpbuf *b, unsigned int match_len, struct match *match,
               ovs_be64 *cookie, ovs_be64 *cookie_mask,
-              const struct tun_table *tun_table,
+              bool pipeline_fields_only, const struct tun_table *tun_table,
               const struct vl_mff_map *vl_mff_map)
 {
-    return nx_pull_match__(b, match_len, true, match, cookie, cookie_mask,
-                           tun_table, vl_mff_map);
+    return nx_pull_match__(b, match_len, true, pipeline_fields_only, match,
+                           cookie, cookie_mask, tun_table, vl_mff_map);
 }
 
 /* Behaves the same as nx_pull_match(), but skips over unknown NXM headers,
  * instead of failing with an error, and does not check for field
- * prerequisities. */
+ * prerequisites. */
 enum ofperr
 nx_pull_match_loose(struct ofpbuf *b, unsigned int match_len,
-                    struct match *match,
-                    ovs_be64 *cookie, ovs_be64 *cookie_mask,
+                    struct match *match, ovs_be64 *cookie,
+                    ovs_be64 *cookie_mask, bool pipeline_fields_only,
                     const struct tun_table *tun_table)
 {
-    return nx_pull_match__(b, match_len, false, match, cookie, cookie_mask,
-                           tun_table, NULL);
+    return nx_pull_match__(b, match_len, false, pipeline_fields_only, match,
+                           cookie, cookie_mask, tun_table, NULL);
 }
 
 static enum ofperr
-oxm_pull_match__(struct ofpbuf *b, bool strict,
+oxm_pull_match__(struct ofpbuf *b, bool strict, bool pipeline_fields_only,
                  const struct tun_table *tun_table,
                  const struct vl_mff_map *vl_mff_map, struct match *match)
 {
@@ -662,11 +669,15 @@ oxm_pull_match__(struct ofpbuf *b, bool strict,
     }
 
     return nx_pull_raw(p + sizeof *omh, match_len - sizeof *omh,
-                       strict, match, NULL, NULL, tun_table, vl_mff_map);
+                       strict, pipeline_fields_only, match, NULL, NULL,
+                       tun_table, vl_mff_map);
 }
 
 /* Parses the oxm formatted match description preceded by a struct
  * ofp11_match_header in 'b'.  Stores the result in 'match'.
+ * If 'pipeline_fields_only' is true, this function returns
+ * OFPERR_OFPBRC_PIPELINE_FIELDS_ONLY if there is any non pipeline fields
+ * in 'b'.
  *
  * 'vl_mff_map' is an optional parameter that is used to validate the length
  * of variable length mf_fields in 'match'. If it is not provided, the
@@ -676,20 +687,23 @@ oxm_pull_match__(struct ofpbuf *b, bool strict,
  *
  * Returns 0 if successful, otherwise an OpenFlow error code. */
 enum ofperr
-oxm_pull_match(struct ofpbuf *b, const struct tun_table *tun_table,
+oxm_pull_match(struct ofpbuf *b, bool pipeline_fields_only,
+               const struct tun_table *tun_table,
                const struct vl_mff_map *vl_mff_map, struct match *match)
 {
-    return oxm_pull_match__(b, true, tun_table, vl_mff_map, match);
+    return oxm_pull_match__(b, true, pipeline_fields_only, tun_table,
+                            vl_mff_map, match);
 }
 
 /* Behaves the same as oxm_pull_match() with two exceptions.  Skips over
  * unknown OXM headers instead of failing with an error when they are
- * encountered, and does not check for field prerequisities. */
+ * encountered, and does not check for field prerequisites. */
 enum ofperr
-oxm_pull_match_loose(struct ofpbuf *b, const struct tun_table *tun_table,
-                     struct match *match)
+oxm_pull_match_loose(struct ofpbuf *b, bool pipeline_fields_only,
+                     const struct tun_table *tun_table, struct match *match)
 {
-    return oxm_pull_match__(b, false, tun_table, NULL, match);
+    return oxm_pull_match__(b, false, pipeline_fields_only, tun_table, NULL,
+                            match);
 }
 
 /* Parses the OXM match description in the 'oxm_len' bytes in 'oxm'.  Stores
@@ -705,8 +719,8 @@ oxm_decode_match(const void *oxm, size_t oxm_len, bool loose,
                  const struct tun_table *tun_table,
                  const struct vl_mff_map *vl_mff_map, struct match *match)
 {
-    return nx_pull_raw(oxm, oxm_len, !loose, match, NULL, NULL, tun_table,
-                       vl_mff_map);
+    return nx_pull_raw(oxm, oxm_len, !loose, false, match, NULL, NULL,
+                       tun_table, vl_mff_map);
 }
 
 /* Verify an array of OXM TLVs treating value of each TLV as a mask,
@@ -760,203 +774,227 @@ oxm_pull_field_array(const void *fields_data, size_t fields_len,
  * 'put' functions whose names end in 'm' add a field that might be wildcarded.
  * Other 'put' functions add exact-match fields.
  */
+
+struct nxm_put_ctx {
+    struct ofpbuf *output;
+    bool implied_ethernet;
+};
+
 void
-nxm_put__(struct ofpbuf *b, enum mf_field_id field, enum ofp_version version,
-          const void *value, const void *mask, size_t n_bytes)
+nxm_put_entry_raw(struct ofpbuf *b,
+                  enum mf_field_id field, enum ofp_version version,
+                  const void *value, const void *mask, size_t n_bytes)
 {
     nx_put_header_len(b, field, version, !!mask, n_bytes);
     ofpbuf_put(b, value, n_bytes);
     if (mask) {
         ofpbuf_put(b, mask, n_bytes);
     }
-
 }
 
 static void
-nxm_put(struct ofpbuf *b, enum mf_field_id field, enum ofp_version version,
-        const void *value, const void *mask, size_t n_bytes)
+nxm_put__(struct nxm_put_ctx *ctx,
+          enum mf_field_id field, enum ofp_version version,
+          const void *value, const void *mask, size_t n_bytes)
 {
-    if (!is_all_zeros(mask, n_bytes)) {
-        bool masked = !is_all_ones(mask, n_bytes);
-        nxm_put__(b, field, version, value, masked ? mask : NULL, n_bytes);
+    nxm_put_entry_raw(ctx->output, field, version, value, mask, n_bytes);
+    if (!ctx->implied_ethernet && mf_from_id(field)->prereqs != MFP_NONE) {
+        ctx->implied_ethernet = true;
     }
 }
 
 static void
-nxm_put_8m(struct ofpbuf *b, enum mf_field_id field, enum ofp_version version,
+nxm_put(struct nxm_put_ctx *ctx,
+        enum mf_field_id field, enum ofp_version version,
+        const void *value, const void *mask, size_t n_bytes)
+{
+    if (!is_all_zeros(mask, n_bytes)) {
+        bool masked = !is_all_ones(mask, n_bytes);
+        nxm_put__(ctx, field, version, value, masked ? mask : NULL, n_bytes);
+    }
+}
+
+static void
+nxm_put_8m(struct nxm_put_ctx *ctx,
+           enum mf_field_id field, enum ofp_version version,
            uint8_t value, uint8_t mask)
 {
-    nxm_put(b, field, version, &value, &mask, sizeof value);
+    nxm_put(ctx, field, version, &value, &mask, sizeof value);
 }
 
 static void
-nxm_put_8(struct ofpbuf *b, enum mf_field_id field, enum ofp_version version,
-          uint8_t value)
+nxm_put_8(struct nxm_put_ctx *ctx,
+          enum mf_field_id field, enum ofp_version version, uint8_t value)
 {
-    nxm_put__(b, field, version, &value, NULL, sizeof value);
+    nxm_put__(ctx, field, version, &value, NULL, sizeof value);
 }
 
 static void
-nxm_put_16m(struct ofpbuf *b, enum mf_field_id field, enum ofp_version version,
+nxm_put_16m(struct nxm_put_ctx *ctx,
+            enum mf_field_id field, enum ofp_version version,
             ovs_be16 value, ovs_be16 mask)
 {
-    nxm_put(b, field, version, &value, &mask, sizeof value);
+    nxm_put(ctx, field, version, &value, &mask, sizeof value);
 }
 
 static void
-nxm_put_16(struct ofpbuf *b, enum mf_field_id field, enum ofp_version version,
-           ovs_be16 value)
+nxm_put_16(struct nxm_put_ctx *ctx,
+           enum mf_field_id field, enum ofp_version version, ovs_be16 value)
 {
-    nxm_put__(b, field, version, &value, NULL, sizeof value);
+    nxm_put__(ctx, field, version, &value, NULL, sizeof value);
 }
 
 static void
-nxm_put_32m(struct ofpbuf *b, enum mf_field_id field, enum ofp_version version,
+nxm_put_32m(struct nxm_put_ctx *ctx,
+            enum mf_field_id field, enum ofp_version version,
             ovs_be32 value, ovs_be32 mask)
 {
-    nxm_put(b, field, version, &value, &mask, sizeof value);
+    nxm_put(ctx, field, version, &value, &mask, sizeof value);
 }
 
 static void
-nxm_put_32(struct ofpbuf *b, enum mf_field_id field, enum ofp_version version,
-           ovs_be32 value)
+nxm_put_32(struct nxm_put_ctx *ctx,
+           enum mf_field_id field, enum ofp_version version, ovs_be32 value)
 {
-    nxm_put__(b, field, version, &value, NULL, sizeof value);
+    nxm_put__(ctx, field, version, &value, NULL, sizeof value);
 }
 
 static void
-nxm_put_64m(struct ofpbuf *b, enum mf_field_id field, enum ofp_version version,
+nxm_put_64m(struct nxm_put_ctx *ctx,
+            enum mf_field_id field, enum ofp_version version,
             ovs_be64 value, ovs_be64 mask)
 {
-    nxm_put(b, field, version, &value, &mask, sizeof value);
+    nxm_put(ctx, field, version, &value, &mask, sizeof value);
 }
 
 static void
-nxm_put_128m(struct ofpbuf *b,
+nxm_put_128m(struct nxm_put_ctx *ctx,
              enum mf_field_id field, enum ofp_version version,
              const ovs_be128 value, const ovs_be128 mask)
 {
-    nxm_put(b, field, version, &value, &mask, sizeof(value));
+    nxm_put(ctx, field, version, &value, &mask, sizeof(value));
 }
 
 static void
-nxm_put_eth_masked(struct ofpbuf *b,
+nxm_put_eth_masked(struct nxm_put_ctx *ctx,
                    enum mf_field_id field, enum ofp_version version,
                    const struct eth_addr value, const struct eth_addr mask)
 {
-    nxm_put(b, field, version, value.ea, mask.ea, ETH_ADDR_LEN);
+    nxm_put(ctx, field, version, value.ea, mask.ea, ETH_ADDR_LEN);
 }
 
 static void
-nxm_put_ipv6(struct ofpbuf *b,
+nxm_put_ipv6(struct nxm_put_ctx *ctx,
              enum mf_field_id field, enum ofp_version version,
              const struct in6_addr *value, const struct in6_addr *mask)
 {
-    nxm_put(b, field, version, value->s6_addr, mask->s6_addr,
+    nxm_put(ctx, field, version, value->s6_addr, mask->s6_addr,
             sizeof value->s6_addr);
 }
 
 static void
-nxm_put_frag(struct ofpbuf *b, const struct match *match,
+nxm_put_frag(struct nxm_put_ctx *ctx, const struct match *match,
              enum ofp_version version)
 {
     uint8_t nw_frag = match->flow.nw_frag & FLOW_NW_FRAG_MASK;
     uint8_t nw_frag_mask = match->wc.masks.nw_frag & FLOW_NW_FRAG_MASK;
 
-    nxm_put_8m(b, MFF_IP_FRAG, version, nw_frag,
+    nxm_put_8m(ctx, MFF_IP_FRAG, version, nw_frag,
                nw_frag_mask == FLOW_NW_FRAG_MASK ? UINT8_MAX : nw_frag_mask);
 }
 
 /* Appends to 'b' a set of OXM or NXM matches for the IPv4 or IPv6 fields in
  * 'match'.  */
 static void
-nxm_put_ip(struct ofpbuf *b, const struct match *match, enum ofp_version oxm)
+nxm_put_ip(struct nxm_put_ctx *ctx,
+           const struct match *match, enum ofp_version oxm)
 {
     const struct flow *flow = &match->flow;
+    ovs_be16 dl_type = get_dl_type(flow);
 
-    if (flow->dl_type == htons(ETH_TYPE_IP)) {
-        nxm_put_32m(b, MFF_IPV4_SRC, oxm,
+    if (dl_type == htons(ETH_TYPE_IP)) {
+        nxm_put_32m(ctx, MFF_IPV4_SRC, oxm,
                     flow->nw_src, match->wc.masks.nw_src);
-        nxm_put_32m(b, MFF_IPV4_DST, oxm,
+        nxm_put_32m(ctx, MFF_IPV4_DST, oxm,
                     flow->nw_dst, match->wc.masks.nw_dst);
     } else {
-        nxm_put_ipv6(b, MFF_IPV6_SRC, oxm,
+        nxm_put_ipv6(ctx, MFF_IPV6_SRC, oxm,
                      &flow->ipv6_src, &match->wc.masks.ipv6_src);
-        nxm_put_ipv6(b, MFF_IPV6_DST, oxm,
+        nxm_put_ipv6(ctx, MFF_IPV6_DST, oxm,
                      &flow->ipv6_dst, &match->wc.masks.ipv6_dst);
     }
 
-    nxm_put_frag(b, match, oxm);
+    nxm_put_frag(ctx, match, oxm);
 
     if (match->wc.masks.nw_tos & IP_DSCP_MASK) {
         if (oxm) {
-            nxm_put_8(b, MFF_IP_DSCP_SHIFTED, oxm,
+            nxm_put_8(ctx, MFF_IP_DSCP_SHIFTED, oxm,
                       flow->nw_tos >> 2);
         } else {
-            nxm_put_8(b, MFF_IP_DSCP, oxm,
+            nxm_put_8(ctx, MFF_IP_DSCP, oxm,
                       flow->nw_tos & IP_DSCP_MASK);
         }
     }
 
     if (match->wc.masks.nw_tos & IP_ECN_MASK) {
-        nxm_put_8(b, MFF_IP_ECN, oxm,
+        nxm_put_8(ctx, MFF_IP_ECN, oxm,
                   flow->nw_tos & IP_ECN_MASK);
     }
 
     if (match->wc.masks.nw_ttl) {
-        nxm_put_8(b, MFF_IP_TTL, oxm, flow->nw_ttl);
+        nxm_put_8(ctx, MFF_IP_TTL, oxm, flow->nw_ttl);
     }
 
-    nxm_put_32m(b, MFF_IPV6_LABEL, oxm,
+    nxm_put_32m(ctx, MFF_IPV6_LABEL, oxm,
                 flow->ipv6_label, match->wc.masks.ipv6_label);
 
     if (match->wc.masks.nw_proto) {
-        nxm_put_8(b, MFF_IP_PROTO, oxm, flow->nw_proto);
+        nxm_put_8(ctx, MFF_IP_PROTO, oxm, flow->nw_proto);
 
         if (flow->nw_proto == IPPROTO_TCP) {
-            nxm_put_16m(b, MFF_TCP_SRC, oxm,
+            nxm_put_16m(ctx, MFF_TCP_SRC, oxm,
                         flow->tp_src, match->wc.masks.tp_src);
-            nxm_put_16m(b, MFF_TCP_DST, oxm,
+            nxm_put_16m(ctx, MFF_TCP_DST, oxm,
                         flow->tp_dst, match->wc.masks.tp_dst);
-            nxm_put_16m(b, MFF_TCP_FLAGS, oxm,
+            nxm_put_16m(ctx, MFF_TCP_FLAGS, oxm,
                         flow->tcp_flags, match->wc.masks.tcp_flags);
         } else if (flow->nw_proto == IPPROTO_UDP) {
-            nxm_put_16m(b, MFF_UDP_SRC, oxm,
+            nxm_put_16m(ctx, MFF_UDP_SRC, oxm,
                         flow->tp_src, match->wc.masks.tp_src);
-            nxm_put_16m(b, MFF_UDP_DST, oxm,
+            nxm_put_16m(ctx, MFF_UDP_DST, oxm,
                         flow->tp_dst, match->wc.masks.tp_dst);
         } else if (flow->nw_proto == IPPROTO_SCTP) {
-            nxm_put_16m(b, MFF_SCTP_SRC, oxm, flow->tp_src,
+            nxm_put_16m(ctx, MFF_SCTP_SRC, oxm, flow->tp_src,
                         match->wc.masks.tp_src);
-            nxm_put_16m(b, MFF_SCTP_DST, oxm, flow->tp_dst,
+            nxm_put_16m(ctx, MFF_SCTP_DST, oxm, flow->tp_dst,
                         match->wc.masks.tp_dst);
         } else if (is_icmpv4(flow, NULL)) {
             if (match->wc.masks.tp_src) {
-                nxm_put_8(b, MFF_ICMPV4_TYPE, oxm,
+                nxm_put_8(ctx, MFF_ICMPV4_TYPE, oxm,
                           ntohs(flow->tp_src));
             }
             if (match->wc.masks.tp_dst) {
-                nxm_put_8(b, MFF_ICMPV4_CODE, oxm,
+                nxm_put_8(ctx, MFF_ICMPV4_CODE, oxm,
                           ntohs(flow->tp_dst));
             }
         } else if (is_icmpv6(flow, NULL)) {
             if (match->wc.masks.tp_src) {
-                nxm_put_8(b, MFF_ICMPV6_TYPE, oxm,
+                nxm_put_8(ctx, MFF_ICMPV6_TYPE, oxm,
                           ntohs(flow->tp_src));
             }
             if (match->wc.masks.tp_dst) {
-                nxm_put_8(b, MFF_ICMPV6_CODE, oxm,
+                nxm_put_8(ctx, MFF_ICMPV6_CODE, oxm,
                           ntohs(flow->tp_dst));
             }
             if (is_nd(flow, NULL)) {
-                nxm_put_ipv6(b, MFF_ND_TARGET, oxm,
+                nxm_put_ipv6(ctx, MFF_ND_TARGET, oxm,
                              &flow->nd_target, &match->wc.masks.nd_target);
                 if (flow->tp_src == htons(ND_NEIGHBOR_SOLICIT)) {
-                    nxm_put_eth_masked(b, MFF_ND_SLL, oxm,
+                    nxm_put_eth_masked(ctx, MFF_ND_SLL, oxm,
                                        flow->arp_sha, match->wc.masks.arp_sha);
                 }
                 if (flow->tp_src == htons(ND_NEIGHBOR_ADVERT)) {
-                    nxm_put_eth_masked(b, MFF_ND_TLL, oxm,
+                    nxm_put_eth_masked(ctx, MFF_ND_TLL, oxm,
                                        flow->arp_tha, match->wc.masks.arp_tha);
                 }
             }
@@ -983,160 +1021,199 @@ nx_put_raw(struct ofpbuf *b, enum ofp_version oxm, const struct match *match,
 {
     const struct flow *flow = &match->flow;
     const size_t start_len = b->size;
+    ovs_be16 dl_type = get_dl_type(flow);
     int match_len;
     int i;
 
-    BUILD_ASSERT_DECL(FLOW_WC_SEQ == 36);
+    BUILD_ASSERT_DECL(FLOW_WC_SEQ == 40);
+
+    struct nxm_put_ctx ctx = { .output = b, .implied_ethernet = false };
+
+    /* OpenFlow Packet Type. Must be first. */
+    if (match->wc.masks.packet_type && !match_has_default_packet_type(match)) {
+        nxm_put_32m(&ctx, MFF_PACKET_TYPE, oxm, flow->packet_type,
+                    match->wc.masks.packet_type);
+    }
 
     /* Metadata. */
     if (match->wc.masks.dp_hash) {
-        nxm_put_32m(b, MFF_DP_HASH, oxm,
+        nxm_put_32m(&ctx, MFF_DP_HASH, oxm,
                     htonl(flow->dp_hash), htonl(match->wc.masks.dp_hash));
     }
 
     if (match->wc.masks.recirc_id) {
-        nxm_put_32(b, MFF_RECIRC_ID, oxm, htonl(flow->recirc_id));
+        nxm_put_32(&ctx, MFF_RECIRC_ID, oxm, htonl(flow->recirc_id));
     }
 
     if (match->wc.masks.conj_id) {
-        nxm_put_32(b, MFF_CONJ_ID, oxm, htonl(flow->conj_id));
+        nxm_put_32(&ctx, MFF_CONJ_ID, oxm, htonl(flow->conj_id));
     }
 
     if (match->wc.masks.in_port.ofp_port) {
         ofp_port_t in_port = flow->in_port.ofp_port;
         if (oxm) {
-            nxm_put_32(b, MFF_IN_PORT_OXM, oxm,
+            nxm_put_32(&ctx, MFF_IN_PORT_OXM, oxm,
                        ofputil_port_to_ofp11(in_port));
         } else {
-            nxm_put_16(b, MFF_IN_PORT, oxm,
+            nxm_put_16(&ctx, MFF_IN_PORT, oxm,
                        htons(ofp_to_u16(in_port)));
         }
     }
     if (match->wc.masks.actset_output) {
-        nxm_put_32(b, MFF_ACTSET_OUTPUT, oxm,
+        nxm_put_32(&ctx, MFF_ACTSET_OUTPUT, oxm,
                    ofputil_port_to_ofp11(flow->actset_output));
     }
 
     /* Ethernet. */
-    nxm_put_eth_masked(b, MFF_ETH_SRC, oxm,
+    nxm_put_eth_masked(&ctx, MFF_ETH_SRC, oxm,
                        flow->dl_src, match->wc.masks.dl_src);
-    nxm_put_eth_masked(b, MFF_ETH_DST, oxm,
+    nxm_put_eth_masked(&ctx, MFF_ETH_DST, oxm,
                        flow->dl_dst, match->wc.masks.dl_dst);
-    nxm_put_16m(b, MFF_ETH_TYPE, oxm,
+    nxm_put_16m(&ctx, MFF_ETH_TYPE, oxm,
                 ofputil_dl_type_to_openflow(flow->dl_type),
                 match->wc.masks.dl_type);
 
     /* 802.1Q. */
     if (oxm) {
         ovs_be16 VID_CFI_MASK = htons(VLAN_VID_MASK | VLAN_CFI);
-        ovs_be16 vid = flow->vlan_tci & VID_CFI_MASK;
-        ovs_be16 mask = match->wc.masks.vlan_tci & VID_CFI_MASK;
+        ovs_be16 vid = flow->vlans[0].tci & VID_CFI_MASK;
+        ovs_be16 mask = match->wc.masks.vlans[0].tci & VID_CFI_MASK;
 
         if (mask == htons(VLAN_VID_MASK | VLAN_CFI)) {
-            nxm_put_16(b, MFF_VLAN_VID, oxm, vid);
+            nxm_put_16(&ctx, MFF_VLAN_VID, oxm, vid);
         } else if (mask) {
-            nxm_put_16m(b, MFF_VLAN_VID, oxm, vid, mask);
+            nxm_put_16m(&ctx, MFF_VLAN_VID, oxm, vid, mask);
         }
 
-        if (vid && vlan_tci_to_pcp(match->wc.masks.vlan_tci)) {
-            nxm_put_8(b, MFF_VLAN_PCP, oxm,
-                      vlan_tci_to_pcp(flow->vlan_tci));
+        if (vid && vlan_tci_to_pcp(match->wc.masks.vlans[0].tci)) {
+            nxm_put_8(&ctx, MFF_VLAN_PCP, oxm,
+                      vlan_tci_to_pcp(flow->vlans[0].tci));
         }
 
     } else {
-        nxm_put_16m(b, MFF_VLAN_TCI, oxm, flow->vlan_tci,
-                    match->wc.masks.vlan_tci);
+        nxm_put_16m(&ctx, MFF_VLAN_TCI, oxm, flow->vlans[0].tci,
+                    match->wc.masks.vlans[0].tci);
     }
 
     /* MPLS. */
-    if (eth_type_mpls(flow->dl_type)) {
+    if (eth_type_mpls(dl_type)) {
         if (match->wc.masks.mpls_lse[0] & htonl(MPLS_TC_MASK)) {
-            nxm_put_8(b, MFF_MPLS_TC, oxm,
+            nxm_put_8(&ctx, MFF_MPLS_TC, oxm,
                       mpls_lse_to_tc(flow->mpls_lse[0]));
         }
 
         if (match->wc.masks.mpls_lse[0] & htonl(MPLS_BOS_MASK)) {
-            nxm_put_8(b, MFF_MPLS_BOS, oxm,
+            nxm_put_8(&ctx, MFF_MPLS_BOS, oxm,
                       mpls_lse_to_bos(flow->mpls_lse[0]));
         }
 
         if (match->wc.masks.mpls_lse[0] & htonl(MPLS_LABEL_MASK)) {
-            nxm_put_32(b, MFF_MPLS_LABEL, oxm,
+            nxm_put_32(&ctx, MFF_MPLS_LABEL, oxm,
                        htonl(mpls_lse_to_label(flow->mpls_lse[0])));
         }
     }
 
     /* L3. */
     if (is_ip_any(flow)) {
-        nxm_put_ip(b, match, oxm);
-    } else if (flow->dl_type == htons(ETH_TYPE_ARP) ||
-               flow->dl_type == htons(ETH_TYPE_RARP)) {
+        nxm_put_ip(&ctx, match, oxm);
+    } else if (dl_type == htons(ETH_TYPE_ARP) ||
+               dl_type == htons(ETH_TYPE_RARP)) {
         /* ARP. */
         if (match->wc.masks.nw_proto) {
-            nxm_put_16(b, MFF_ARP_OP, oxm,
+            nxm_put_16(&ctx, MFF_ARP_OP, oxm,
                        htons(flow->nw_proto));
         }
-        nxm_put_32m(b, MFF_ARP_SPA, oxm,
+        nxm_put_32m(&ctx, MFF_ARP_SPA, oxm,
                     flow->nw_src, match->wc.masks.nw_src);
-        nxm_put_32m(b, MFF_ARP_TPA, oxm,
+        nxm_put_32m(&ctx, MFF_ARP_TPA, oxm,
                     flow->nw_dst, match->wc.masks.nw_dst);
-        nxm_put_eth_masked(b, MFF_ARP_SHA, oxm,
+        nxm_put_eth_masked(&ctx, MFF_ARP_SHA, oxm,
                            flow->arp_sha, match->wc.masks.arp_sha);
-        nxm_put_eth_masked(b, MFF_ARP_THA, oxm,
+        nxm_put_eth_masked(&ctx, MFF_ARP_THA, oxm,
                            flow->arp_tha, match->wc.masks.arp_tha);
     }
 
     /* Tunnel ID. */
-    nxm_put_64m(b, MFF_TUN_ID, oxm,
+    nxm_put_64m(&ctx, MFF_TUN_ID, oxm,
                 flow->tunnel.tun_id, match->wc.masks.tunnel.tun_id);
 
     /* Other tunnel metadata. */
-    nxm_put_16m(b, MFF_TUN_FLAGS, oxm,
+    nxm_put_16m(&ctx, MFF_TUN_FLAGS, oxm,
                 htons(flow->tunnel.flags), htons(match->wc.masks.tunnel.flags));
-    nxm_put_32m(b, MFF_TUN_SRC, oxm,
+    nxm_put_32m(&ctx, MFF_TUN_SRC, oxm,
                 flow->tunnel.ip_src, match->wc.masks.tunnel.ip_src);
-    nxm_put_32m(b, MFF_TUN_DST, oxm,
+    nxm_put_32m(&ctx, MFF_TUN_DST, oxm,
                 flow->tunnel.ip_dst, match->wc.masks.tunnel.ip_dst);
-    nxm_put_ipv6(b, MFF_TUN_IPV6_SRC, oxm,
+    nxm_put_ipv6(&ctx, MFF_TUN_IPV6_SRC, oxm,
                  &flow->tunnel.ipv6_src, &match->wc.masks.tunnel.ipv6_src);
-    nxm_put_ipv6(b, MFF_TUN_IPV6_DST, oxm,
+    nxm_put_ipv6(&ctx, MFF_TUN_IPV6_DST, oxm,
                  &flow->tunnel.ipv6_dst, &match->wc.masks.tunnel.ipv6_dst);
-    nxm_put_16m(b, MFF_TUN_GBP_ID, oxm,
+    nxm_put_16m(&ctx, MFF_TUN_GBP_ID, oxm,
                 flow->tunnel.gbp_id, match->wc.masks.tunnel.gbp_id);
-    nxm_put_8m(b, MFF_TUN_GBP_FLAGS, oxm,
+    nxm_put_8m(&ctx, MFF_TUN_GBP_FLAGS, oxm,
                flow->tunnel.gbp_flags, match->wc.masks.tunnel.gbp_flags);
     tun_metadata_to_nx_match(b, oxm, match);
 
+    /* Network Service Header */
+    nxm_put_8m(&ctx, MFF_NSH_FLAGS, oxm, flow->nsh.flags,
+            match->wc.masks.nsh.flags);
+    nxm_put_8m(&ctx, MFF_NSH_MDTYPE, oxm, flow->nsh.mdtype,
+            match->wc.masks.nsh.mdtype);
+    nxm_put_8m(&ctx, MFF_NSH_NP, oxm, flow->nsh.np,
+            match->wc.masks.nsh.np);
+    nxm_put_32m(&ctx, MFF_NSH_SPI, oxm, flow->nsh.spi,
+                match->wc.masks.nsh.spi);
+    nxm_put_8m(&ctx, MFF_NSH_SI, oxm, flow->nsh.si, match->wc.masks.nsh.si);
+    for (int i = 0; i < 4; i++) {
+        nxm_put_32m(&ctx, MFF_NSH_C1 + i, oxm, flow->nsh.c[i],
+                    match->wc.masks.nsh.c[i]);
+    }
+
     /* Registers. */
     if (oxm < OFP15_VERSION) {
         for (i = 0; i < FLOW_N_REGS; i++) {
-            nxm_put_32m(b, MFF_REG0 + i, oxm,
+            nxm_put_32m(&ctx, MFF_REG0 + i, oxm,
                         htonl(flow->regs[i]), htonl(match->wc.masks.regs[i]));
         }
     } else {
         for (i = 0; i < FLOW_N_XREGS; i++) {
-            nxm_put_64m(b, MFF_XREG0 + i, oxm,
+            nxm_put_64m(&ctx, MFF_XREG0 + i, oxm,
                         htonll(flow_get_xreg(flow, i)),
                         htonll(flow_get_xreg(&match->wc.masks, i)));
         }
     }
 
     /* Packet mark. */
-    nxm_put_32m(b, MFF_PKT_MARK, oxm, htonl(flow->pkt_mark),
+    nxm_put_32m(&ctx, MFF_PKT_MARK, oxm, htonl(flow->pkt_mark),
                 htonl(match->wc.masks.pkt_mark));
 
     /* Connection tracking. */
-    nxm_put_32m(b, MFF_CT_STATE, oxm, htonl(flow->ct_state),
+    nxm_put_32m(&ctx, MFF_CT_STATE, oxm, htonl(flow->ct_state),
                 htonl(match->wc.masks.ct_state));
-    nxm_put_16m(b, MFF_CT_ZONE, oxm, htons(flow->ct_zone),
+    nxm_put_16m(&ctx, MFF_CT_ZONE, oxm, htons(flow->ct_zone),
                 htons(match->wc.masks.ct_zone));
-    nxm_put_32m(b, MFF_CT_MARK, oxm, htonl(flow->ct_mark),
+    nxm_put_32m(&ctx, MFF_CT_MARK, oxm, htonl(flow->ct_mark),
                 htonl(match->wc.masks.ct_mark));
-    nxm_put_128m(b, MFF_CT_LABEL, oxm, hton128(flow->ct_label),
+    nxm_put_128m(&ctx, MFF_CT_LABEL, oxm, hton128(flow->ct_label),
                  hton128(match->wc.masks.ct_label));
-
+    nxm_put_32m(&ctx, MFF_CT_NW_SRC, oxm,
+                flow->ct_nw_src, match->wc.masks.ct_nw_src);
+    nxm_put_ipv6(&ctx, MFF_CT_IPV6_SRC, oxm,
+                 &flow->ct_ipv6_src, &match->wc.masks.ct_ipv6_src);
+    nxm_put_32m(&ctx, MFF_CT_NW_DST, oxm,
+                flow->ct_nw_dst, match->wc.masks.ct_nw_dst);
+    nxm_put_ipv6(&ctx, MFF_CT_IPV6_DST, oxm,
+                 &flow->ct_ipv6_dst, &match->wc.masks.ct_ipv6_dst);
+    if (flow->ct_nw_proto) {
+        nxm_put_8m(&ctx, MFF_CT_NW_PROTO, oxm, flow->ct_nw_proto,
+                   match->wc.masks.ct_nw_proto);
+        nxm_put_16m(&ctx, MFF_CT_TP_SRC, oxm,
+                    flow->ct_tp_src, match->wc.masks.ct_tp_src);
+        nxm_put_16m(&ctx, MFF_CT_TP_DST, oxm,
+                    flow->ct_tp_dst, match->wc.masks.ct_tp_dst);
+    }
     /* OpenFlow 1.1+ Metadata. */
-    nxm_put_64m(b, MFF_METADATA, oxm,
+    nxm_put_64m(&ctx, MFF_METADATA, oxm,
                 flow->metadata, match->wc.masks.metadata);
 
     /* Cookie. */
@@ -1151,6 +1228,16 @@ nx_put_raw(struct ofpbuf *b, enum ofp_version oxm, const struct match *match,
         }
     }
 
+    if (match_has_default_packet_type(match) && !ctx.implied_ethernet) {
+        uint64_t pt_stub[16 / 8];
+        struct ofpbuf pt;
+        ofpbuf_use_stack(&pt, pt_stub, sizeof pt_stub);
+        nxm_put_entry_raw(&pt, MFF_PACKET_TYPE, oxm, &flow->packet_type,
+                          NULL, sizeof flow->packet_type);
+
+        ofpbuf_insert(b, start_len, pt.data, pt.size);
+    }
+
     match_len = b->size - start_len;
     return match_len;
 }
@@ -1234,7 +1321,7 @@ nx_format_mask_tlv(struct ds *ds, enum mf_field_id id,
 
     if (!is_all_ones(mask, mf->n_bytes)) {
         ds_put_char(ds, '=');
-        mf_format(mf, mask, NULL, ds);
+        mf_format(mf, mask, NULL, NULL, ds);
     }
 
     ds_put_char(ds, ',');
@@ -1300,7 +1387,8 @@ oxm_put_field_array(struct ofpbuf *b, const struct field_array *fa,
         memcpy(&value, fa->values + offset, mf->n_bytes);
 
         int len = mf_field_len(mf, &value, NULL, NULL);
-        nxm_put__(b, i, version, &value + mf->n_bytes - len, NULL, len);
+        nxm_put_entry_raw(b, i, version,
+                          &value + mf->n_bytes - len, NULL, len);
         offset += mf->n_bytes;
     }
 
@@ -1357,11 +1445,9 @@ nx_put_entry(struct ofpbuf *b, const struct mf_field *mff,
     len = mf_field_len(mff, value, mask, &masked);
     offset = mff->n_bytes - len;
 
-    nx_put_header_len(b, mff->id, version, masked, len);
-    ofpbuf_put(b, &value->u8 + offset, len);
-    if (masked) {
-        ofpbuf_put(b, &mask->u8 + offset, len);
-    }
+    nxm_put_entry_raw(b, mff->id, version,
+                      &value->u8 + offset, masked ? &mask->u8 + offset : NULL,
+                      len);
 }
 
 /* nx_match_to_string() and helpers. */
@@ -1686,16 +1772,17 @@ nxm_format_reg_move(const struct ofpact_reg_move *move, struct ds *s)
 
 
 enum ofperr
-nxm_reg_move_check(const struct ofpact_reg_move *move, const struct flow *flow)
+nxm_reg_move_check(const struct ofpact_reg_move *move,
+                   const struct match *match)
 {
     enum ofperr error;
 
-    error = mf_check_src(&move->src, flow);
+    error = mf_check_src(&move->src, match);
     if (error) {
         return error;
     }
 
-    return mf_check_dst(&move->dst, flow);
+    return mf_check_dst(&move->dst, match);
 }
 
 /* nxm_execute_reg_move(). */
@@ -1757,16 +1844,16 @@ nxm_format_stack_pop(const struct ofpact_stack *pop, struct ds *s)
 
 enum ofperr
 nxm_stack_push_check(const struct ofpact_stack *push,
-                     const struct flow *flow)
+                     const struct match *match)
 {
-    return mf_check_src(&push->subfield, flow);
+    return mf_check_src(&push->subfield, match);
 }
 
 enum ofperr
 nxm_stack_pop_check(const struct ofpact_stack *pop,
-                    const struct flow *flow)
+                    const struct match *match)
 {
-    return mf_check_dst(&pop->subfield, flow);
+    return mf_check_dst(&pop->subfield, match);
 }
 
 /* nxm_execute_stack_push(), nxm_execute_stack_pop().
