@@ -23,21 +23,21 @@
 #include <stdlib.h>
 #include <string.h>
 #include "coverage.h"
-#include "dynamic-string.h"
 #include "fatal-signal.h"
 #include "flow.h"
 #include "jsonrpc.h"
-#include "ofp-print.h"
-#include "ofpbuf.h"
 #include "openflow/nicira-ext.h"
 #include "openflow/openflow.h"
+#include "openvswitch/dynamic-string.h"
+#include "openvswitch/ofp-print.h"
+#include "openvswitch/ofpbuf.h"
+#include "openvswitch/vlog.h"
 #include "ovs-thread.h"
 #include "packets.h"
 #include "poll-loop.h"
 #include "random.h"
 #include "socket-util.h"
 #include "util.h"
-#include "openvswitch/vlog.h"
 
 VLOG_DEFINE_THIS_MODULE(stream);
 
@@ -156,6 +156,9 @@ stream_usage(const char *name, bool active, bool passive,
         printf("  --bootstrap-ca-cert=FILE  file with peer CA certificate "
                "to read or create\n");
     }
+    printf("SSL options:\n"
+           "  --ssl-protocols=PROTOS  list of SSL protocols to enable\n"
+           "  --ssl-ciphers=CIPHERS   list of SSL ciphers to enable\n");
 #endif
 }
 
@@ -278,8 +281,10 @@ stream_close(struct stream *stream)
 {
     if (stream != NULL) {
         char *name = stream->name;
+        char *peer_id = stream->peer_id;
         (stream->class->close)(stream);
         free(name);
+        free(peer_id);
     }
 }
 
@@ -430,6 +435,19 @@ stream_send_wait(struct stream *stream)
     stream_wait(stream, STREAM_SEND);
 }
 
+void
+stream_set_peer_id(struct stream *stream, const char *peer_id)
+{
+    free(stream->peer_id);
+    stream->peer_id = xstrdup(peer_id);
+}
+
+const char *
+stream_get_peer_id(const struct stream *stream)
+{
+    return stream->peer_id;
+}
+
 /* Given 'name', a pstream name in the form "TYPE:ARGS", stores the class
  * named "TYPE" into '*classp' and returns 0.  Returns EAFNOSUPPORT and stores
  * a null pointer into '*classp' if 'name' is in the wrong form or if no such
@@ -616,10 +634,10 @@ pstream_get_bound_port(const struct pstream *pstream)
  * After calling this function, stream_close() must be used to destroy
  * 'stream', otherwise resources will be leaked.
  *
- * The caller retains ownership of 'name'. */
+ * Takes ownership of 'name'. */
 void
 stream_init(struct stream *stream, const struct stream_class *class,
-            int connect_status, const char *name)
+            int connect_status, char *name)
 {
     memset(stream, 0, sizeof *stream);
     stream->class = class;
@@ -627,17 +645,18 @@ stream_init(struct stream *stream, const struct stream_class *class,
                     : !connect_status ? SCS_CONNECTED
                     : SCS_DISCONNECTED);
     stream->error = connect_status;
-    stream->name = xstrdup(name);
+    stream->name = name;
     ovs_assert(stream->state != SCS_CONNECTING || class->connect);
 }
 
+/* Takes ownership of 'name'. */
 void
 pstream_init(struct pstream *pstream, const struct pstream_class *class,
-            const char *name)
+            char *name)
 {
     memset(pstream, 0, sizeof *pstream);
     pstream->class = class;
-    pstream->name = xstrdup(name);
+    pstream->name = name;
 }
 
 void
